"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var ts_smart_logger_1 = require("ts-smart-logger");
var util_1 = require("../../util");
var universal_connector_action_builder_1 = require("./universal-connector-action.builder");
var logger = ts_smart_logger_1.LoggerFactory.makeLogger('vue-connector.factory');
/**
 * @stable [23.10.2018]
 * @param {IVueConnectorOptionsConfigEntity<TApplicationStoreEntity extends IVueApplicationStoreEntity>} config
 * @returns {VueComponentOptionsT<IVueContainer<TApplicationStoreEntity extends IVueApplicationStoreEntity>>}
 */
exports.vueConnectorOptionsFactory = function (config) {
    var storeUnsubscriber;
    var reduxStoreListeners = [];
    var customComputed = config.customComputed$;
    if (Array.isArray(customComputed)) {
        customComputed.forEach(function (mapper) { return reduxStoreListeners.push(mapper); });
    }
    return {
        /**
         * @stable [21.10.2018]
         */
        components: config.components,
        computed: config.computed,
        template: config.template,
        watch: config.watch,
        beforeUpdate: config.beforeUpdate,
        updated: config.updated,
        methods: config.methods,
        /**
         * @stable [21.10.2018]
         */
        data: function () {
            var _this = this;
            var initialData = {};
            reduxStoreListeners.forEach(function (mapper) { return Object.assign(initialData, mapper(_this.state$)); });
            return __assign({}, (util_1.isFn(config.data) ? config.data.call(this) : {}), initialData);
        },
        /**
         * @stable [21.10.2018]
         */
        beforeCreate: function () {
            this.sectionName = config.sectionName;
        },
        /**
         * @stable [21.10.2018]
         */
        mounted: function () {
            var _this = this;
            logger.debug("[$vueConnectorOptionsFactory][mounted] Section: " + config.sectionName);
            var self = this;
            var store = self.store$;
            if (util_1.isFn(config.mounted)) {
                config.mounted();
            }
            if (util_1.isFn(self.onMount$)) {
                self.onMount$();
            }
            // Send an init action
            store.dispatch({ type: universal_connector_action_builder_1.UniversalConnectorActionBuilder.buildInitActionType(self.sectionName) });
            var cachedReduxLinks = new Map();
            // Subscribe
            storeUnsubscriber = store.subscribe(function () {
                var state = self.state$;
                /**
                 * TODO Make unit-tests
                 * A "Redux <-> Container" bridge implemented here.
                 * Vue is updated when the data is updated via assign new links
                 */
                reduxStoreListeners.forEach(function (shallowMapper) {
                    var reduxData = shallowMapper(state);
                    var needToUpdateReactiveLinks = [];
                    R.forEachObjIndexed(function (primitiveValueOrLinkToComplexObject, mappedKey) {
                        var cachedReduxLinksByMapper = cachedReduxLinks.get(shallowMapper);
                        if (!R.isNil(cachedReduxLinksByMapper)) {
                            if ((mappedKey in cachedReduxLinksByMapper)) {
                                /**
                                 * Here we compare links to complex objects or primitives (without deep copy!).
                                 * Reducers change the links to objects, therefore, we should compare only links.
                                 */
                                if (cachedReduxLinksByMapper[mappedKey] !== primitiveValueOrLinkToComplexObject) {
                                    needToUpdateReactiveLinks.push(mappedKey);
                                }
                            }
                            else {
                                needToUpdateReactiveLinks.push(mappedKey);
                            }
                        }
                        else {
                            cachedReduxLinks.set(shallowMapper, cachedReduxLinksByMapper = Object.create(null));
                            needToUpdateReactiveLinks.push(mappedKey);
                        }
                        cachedReduxLinksByMapper[mappedKey] = primitiveValueOrLinkToComplexObject;
                    }, reduxData);
                    if (needToUpdateReactiveLinks.length > 0) {
                        // Need to set and shallow clone reactive data only when a snapshot was changed because of:
                        // 1. performance
                        // 2. vue does replace recursively all links to reactive (except for arrays)
                        needToUpdateReactiveLinks.forEach(function (linkToUpdate) {
                            // Don't deep copy !! Vue does not replace an arrays (big arrays) !! It's good for performance
                            self[linkToUpdate] = util_1.shallowClone(reduxData[linkToUpdate]);
                            logger.debug("[$vueConnectorOptionsFactory][mounted] The reactive link for container " + _this.$vnode.tag + " has been updated! Updated link:", linkToUpdate);
                        });
                    }
                    if (util_1.isFn(config.forceUpdateOnChangeData$)) {
                        var previousValue = cachedReduxLinks.get(config.forceUpdateOnChangeData$);
                        var currentValue = config.forceUpdateOnChangeData$(state);
                        if (!R.isNil(currentValue) && previousValue !== currentValue) {
                            self.$forceUpdate();
                            cachedReduxLinks.set(config.forceUpdateOnChangeData$, currentValue);
                            logger.debug("[$vueConnectorOptionsFactory][mounted] The \"forceUpdate\" value has been changed => need to call $forceUpdate. " + _this.$vnode.tag);
                        }
                    }
                });
            });
        },
        /**
         * @stable [21.10.2018]
         */
        beforeDestroy: function () {
            if (util_1.isFn(storeUnsubscriber)) {
                storeUnsubscriber();
            }
            if (util_1.isFn(config.beforeDestroy)) {
                config.beforeDestroy();
            }
        },
        /**
         * @stable [21.01.2019]
         */
        destroyed: function () {
            var self = this;
            var store = self.store$;
            // Send an destroy action
            store.dispatch({ type: universal_connector_action_builder_1.UniversalConnectorActionBuilder.buildDestroyActionType(self.sectionName) });
        },
    };
};
//# sourceMappingURL=vue-connector.factory.js.map